package de.ozgcloud.nachrichten.postfach.osiv2;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.time.OffsetDateTime;
import java.util.Arrays;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;
import org.springframework.test.context.TestPropertySource;

import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.ResponseDefinitionBuilder;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.Jwt;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.OsiMockServerExtension;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.JsonUtil;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.MessageExchangeReceiveMessageTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.MessageExchangeReceiveMessagesResponseTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.MessageExchangeSendMessageResponseTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.V1ReplyMessageTestFactory;
import lombok.SneakyThrows;

@SpringBootTest(classes = TestApplication.class)
@ActiveProfiles({ "stage", "itcase" })
@TestPropertySource(properties = {
		"ozgcloud.osiv2.proxy.enabled=false",
})
class OsiPostfachRemoteServiceITCase {

	@RegisterExtension
	static final OsiMockServerExtension OSI_MOCK_SERVER_EXTENSION = new OsiMockServerExtension();

	private final PostfachNachricht postfachNachricht = PostfachNachrichtTestFactory.create();

	@Autowired
	private OsiPostfachRemoteService osiPostfachRemoteService;

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add("spring.security.oauth2.client.provider.osi2.token-uri", OSI_MOCK_SERVER_EXTENSION::getAccessTokenUrl);
		registry.add("spring.security.oauth2.client.registration.osi2.scope", () -> CLIENT_SCOPES);
		registry.add("spring.security.oauth2.client.registration.osi2.client-id", () -> CLIENT_ID);
		registry.add("ozgcloud.osiv2.api.url", OSI_MOCK_SERVER_EXTENSION::getPostfachFacadeUrl);
		registry.add("ozgcloud.osiv2.api.resource", () -> RESOURCE_URN);
	}

	private WireMockServer postfachFacadeMockServer;

	@BeforeEach
	@SneakyThrows
	public void setup() {
		postfachFacadeMockServer = OSI_MOCK_SERVER_EXTENSION.getPostfachFacadeMockServer();
	}

	@DisplayName("should send request with jwt")
	@Test
	@SneakyThrows
	void shouldSendRequestWithJwt() {
		// Stub message send response (MessageExchangeApi::sendMessage)
		postfachFacadeMockServer.stubFor(post(urlPathTemplate("/MessageExchange/v1/Send/{mailboxId}"))
				.willReturn(
						okJsonObj(
								MessageExchangeSendMessageResponseTestFactory.create()
										.messageId(UUID.randomUUID())
						)
				)
		);

		osiPostfachRemoteService.sendMessage(postfachNachricht);

		var requests = postfachFacadeMockServer.findAll(
				postRequestedFor(urlPathTemplate("/MessageExchange/v1/Send/{mailboxId}")));
		assertThat(requests).hasSize(1);
		var jwt = Jwt.parseAuthHeaderValue(requests.getFirst().getHeader("Authorization"));
		assertThat(jwt.body().read("$.client_id", String.class)).isEqualTo(CLIENT_ID);
		assertThat(jwt.body().read("$.aud", String.class)).isEqualTo(RESOURCE_URN);
	}

	@DisplayName("should receive one messages")
	@Test
	@SneakyThrows
	void shouldReceiveMessages() {
		mockPostfachMessageAndResponse("00000000-0000-0000-0000-000000000000");

		var messageList = osiPostfachRemoteService.getAllMessages().toList();

		assertThat(messageList).hasSize(1);
	}

	@DisplayName("should receive two messages")
	@Test
	@SneakyThrows
	void shouldReceiveTwoMessages() {
		mockPostfachMessageAndResponse("00000000-0000-0000-0000-000000000000", "00000000-0000-0000-0000-000000000001");

		var messageList = osiPostfachRemoteService.getAllMessages().toList();

		assertThat(messageList).hasSize(2);
	}

	private void mockPostfachMessageAndResponse(final String... uuids) {
		// Stub message listing response (MessageExchangeApi::receiveMessages)
		postfachFacadeMockServer.stubFor(get(urlPathEqualTo("/MessageExchange/v1/Receive"))
				.withQueryParam("take", equalTo("100"))
				.withQueryParam("skip", equalTo("0"))
				.willReturn(
						okJsonObj(
								MessageExchangeReceiveMessagesResponseTestFactory.create()
										.messages(Arrays.stream(uuids)
												.map(uuid -> MessageExchangeReceiveMessageTestFactory.create()
														.guid(UUID.fromString(uuid)))
												.toList())
						)
				)
		);
		for (String uuid : uuids) {
			// Stub individual response for message (MessageExchangeApi::getMessage)
			postfachFacadeMockServer.stubFor(get(urlPathTemplate("/MessageExchange/v1/Receive/{messageId}"))
					.withPathParam("messageId", equalTo(uuid))
					.willReturn(
							okJsonObj(
									V1ReplyMessageTestFactory.create()
											.messageBox(UUID.fromString(uuid))
											.responseTime(OffsetDateTime.now())
							)
					)
			);
		}
	}

	private ResponseDefinitionBuilder okJsonObj(final Object body) {
		return okJson(JsonUtil.toJson(body));
	}

	@DisplayName("should delete message")
	@Test
	@SneakyThrows
	void shouldDeleteMessage() {
		var messageId = "00000000-0000-0000-0000-000000000000";
		// Stub delete message response (MessageExchangeApi::deleteMessage)
		postfachFacadeMockServer.stubFor(delete(urlPathTemplate("/MessageExchange/v1/Delete/{messageId}"))
				.willReturn(ok())
		);

		osiPostfachRemoteService.deleteMessage(messageId);

		postfachFacadeMockServer.verify(
				exactly(1),
				deleteRequestedFor(urlPathTemplate("/MessageExchange/v1/Delete/{messageId}"))
						.withPathParam("messageId", equalTo(messageId))
		);
	}

}
